using System;
using System.Collections.Generic;
using System.Text;

namespace Jenyay.Mathematics
{
	/// <summary>
	///    . ,         0
	/// </summary>
	public class Spline: IApproximation
	{
		/// <summary>
		///   X
		/// </summary>
		double[] _x;

		/// <summary>
		///     _x
		/// </summary>
		double[] _y;

		/// <summary>
		///   .
		/// </summary>
		double[] _m;

		public double[] Derivative
		{
			get { return _m; }
		}

		/// <summary>
		///  
		/// </summary>
		/// <param name="x">  X</param>
		/// <param name="y">    X</param>
		/// <remarks>  X     Y.         .  X     .</remarks>
		public Spline (double[] x, double[] y)
		{
			if (x.Length != y.Length)
			{
				throw new ArgumentException ();
			}

			if (x.Length < 3)
			{
				throw new ArgumentException ();
			}

			// 1
			_x = x;
			_y = y;

			CalcDerivative ();
		}

		/// <summary>
		///    
		/// </summary>
		/// <param name="xpoint">  X,     </param>
		/// <returns></returns>
		public double GetValue (double xpoint)
		{
			if (xpoint > _x[_x.Length - 1])
			{
				return ExtrapolateRight (xpoint);
			}

			if (xpoint <= _x[0])
			{
				return ExtrapolateLeft (xpoint);
			}

			return Interpolate (xpoint);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="xpoint"></param>
		/// <returns></returns>
		private double Interpolate (double xpoint)
		{
			int i = 0;
			for (; xpoint > _x[i]; i++) ;

			int j = i - 1;
			double d = _x[i] - _x[j];
			double h = xpoint - _x[j];
			double r = _x[i] - xpoint;
			double p = d * d / 6;
			double y = (_m[j] * r * r * r + _m[i] * h * h * h) / (6 * d);
			y += ((_y[j] - _m[j] * p) * r + (_y[i] - _m[i] * p) * h) / d;

			return y;
		}

		/// <summary>
		///  
		/// </summary>
		/// <param name="xpoint"></param>
		/// <returns></returns>
		private double ExtrapolateRight (double xpoint)
		{
			int n = _x.Length;

			double d = _x[n - 1] - _x[n - 2];
			double y = d * _m[n - 2] / 6 + (_y[n - 1] - _y[n - 2]) / d;
			y = y * (xpoint - _x[n - 1]) + _y[n - 1];

			return y;
		}

		/// <summary>
		///  
		/// </summary>
		/// <param name="xpoint"></param>
		/// <returns></returns>
		private double ExtrapolateLeft (double xpoint)
		{
			double d = _x[1] - _x[0];
			double y = -d * _m[1] / 6 + (_y[1] - _y[0]) / d;
			y = y * (xpoint - _x[0]) + _y[0];

			return y;
		}

		/// <summary>
		///   
		/// </summary>
		private void CalcDerivative ()
		{
			int n = _x.Length;

			_m = new double[n];

			double[] l = new double[n];
			double[] r = new double[n];
			double[] s = new double[n];

			// 2
			double d = _x[1] - _x[0];
			double e = (_y[1] - _y[0]) / d;

			// 3
			for (int k = 1; k < n - 1; k++)
			{
				double h = d;
				d = _x[k + 1] - _x[k];

				double f = e;
				e = (_y[k + 1] - _y[k]) / d;

				l[k] = d / (d + h);
				r[k] = 1 - l[k];
				s[k] = 6 * (e - f) / (h + d);
			}

			// 4
			for (int k = 1; k < n - 1; k++)
			{
				double p = 1.0 / (r[k] * l[k - 1] + 2);
				l[k] *= -p;
				s[k] = (s[k] - r[k] * s[k - 1]) * p;
			}

			// 5
			_m[n - 1] = 0;
			l[n - 2] = s[n - 2];
			_m[n - 2] = l[n - 2];

			// 6
			for (int k = n - 3; k >= 0; k--)
			{
				l[k] = l[k] * l[k + 1] + s[k];
				_m[k] = l[k];
			}
		}

	}
}
