using System;

namespace Jenyay.Mathematics
{
	/// <summary>
	///         
	/// </summary>
	public static class Fourier
	{
		/// <summary>
		///   
		/// </summary>
		/// <param name="input"> </param>
		/// <param name="nx">    x</param>
		/// <returns></returns>
		static public Complex[,] FFT2 (double[,] input, int nx)
		{
			bool inverse = false;

			//  double[,]  Complex[,]
			int ny = input.Length / nx;

			Complex[,] cinput = new Complex[nx, ny];

			for (int ix = 0; ix < nx; ++ix)
			{
				for (int iy = 0; iy < ny; ++iy)
				{
					cinput[ix, iy] = input[ix, iy];
				}
			}

			Complex[,] res = FFT2Body (cinput, nx, inverse);

			return res;
		}

		/// <summary>
		///   
		/// </summary>
		/// <param name="input"> </param>
		/// <param name="nx">    x</param>
		/// <returns></returns>
		static public Complex[,] DFT2 (double[,] input, int nx)
		{
			bool inverse = false;

			//  double[,]  Complex[,]
			int ny = input.Length / nx;

			Complex[,] cinput = new Complex[nx, ny];

			for (int ix = 0; ix < nx; ++ix)
			{
				for (int iy = 0; iy < ny; ++iy)
				{
					cinput[ix, iy] = input[ix, iy];
				}
			}

			Complex[,] res = DFT2Body(cinput, nx, inverse);

			return res;
		}

		/// <summary>
		///   
		/// </summary>
		/// <param name="input"> </param>
		/// <param name="nx">    x</param>
		/// <returns></returns>
		static public Complex[,] IDFT2 (double[,] input, int nx)
		{
			bool inverse = true;

			//  double[,]  Complex[,]
			int ny = input.Length / nx;

			Complex[,] cinput = new Complex[nx, ny];

			for (int ix = 0; ix < nx; ++ix)
			{
				for (int iy = 0; iy < ny; ++iy)
				{
					cinput[ix, iy] = input[ix, iy];
				}
			}

			Complex[,] res = DFT2Body(cinput, nx, inverse);

			return res;
		}

		/// <summary>
		///   
		/// </summary>
		/// <param name="input"> </param>
		/// <param name="nx">    x</param>
		/// <returns></returns>
		static public Complex[,] DFT2 (Complex[,] input, int nx)
		{
			bool inverse = false;

			Complex[,] res = DFT2Body(input, nx, inverse);

			return res;
		}

		/// <summary>
		///   
		/// </summary>
		/// <param name="input"> </param>
		/// <param name="nx">    x</param>
		/// <returns></returns>
		static public Complex[,] IDFT2 (Complex[,] input, int nx)
		{
			bool inverse = true;

			Complex[,] res = DFT2Body(input, nx, inverse);

			return res;
		}

		/// <summary>
		///   
		/// </summary>
		/// <param name="input"> </param>
		/// <param name="nx">    x</param>
		/// <returns></returns>
		static public Complex[,] FFT2(Complex[,] input, int nx)
		{
			bool inverse = false;

			Complex[,] res = FFT2Body (input, nx, inverse);

			return res;
		}

		/// <summary>
		///    
		/// </summary>
		/// <param name="input"> </param>
		/// <param name="nx">    x</param>
		/// <returns></returns>
		static public Complex[,] IFFT2(Complex[,] input, int nx)
		{
			bool inverse = true;

			Complex[,] res = FFT2Body (input, nx, inverse);

			return res;
		}

		/// <summary>
		///    
		/// </summary>
		/// <param name="input"> </param>
		/// <param name="nx">    x</param>
		/// <returns></returns>
		static public Complex[,] IFFT2(double[,] input, int nx)
		{
			bool inverse = true;

			//  double[,]  Complex[,]
			int ny = input.Length / nx;

			Complex[,] cinput = new Complex[nx, ny];

			for (int ix = 0; ix < nx; ++ix)
			{
				for (int iy = 0; iy < ny; ++iy)
				{
					cinput[ix, iy] = input[ix, iy];
				}
			}

			Complex[,] res = FFT2Body (cinput, nx, inverse);

			return res;
		}

		private static Complex[,] DFT2Body (Complex[,] input, int nx, bool inverse)
		{
			int ny = input.Length / nx;

			int k = inverse ? 1 : -1;

			Complex[,] res = new Complex[nx, ny];

			for (int ix = 0; ix < nx; ++ix)
			{
				for (int iy = 0; iy < ny; ++iy)
				{
					res[ix, iy] = 0;

					for (int jx = 0; jx < nx; ++jx)
					{
						for (int jy = 0; jy < ny; ++jy)
						{
							res[ix, iy] += input [jx, jy] * 
								Complex.Exp (k * 2 * Math.PI * Complex.J * (
								(double)(ix * jx) / nx + (double)(iy * jy) / ny) );
						}
					}

					if (inverse)
					{
						res[ix, iy] /= (nx * ny);
					}
				}
			}
			
			return res;
		}

		private static Complex[,] FFT2Body(Complex[,] input, int nx, bool inverse)
		{
			int ny = input.Length / nx;

			Complex[,] res = new Complex[nx, ny];

			for (int ix = 0; ix < nx; ++ix)
			{
				for (int iy = 0; iy < ny; ++iy)
				{
					res[ix, iy] = input[ix, iy];
				}
			}

			//    X
			Complex[] tmpx = new Complex[nx];
			for (int iy = 0; iy < ny; ++iy)
			{
				for (int ix = 0; ix < nx; ++ix)
				{
					tmpx[ix] = res[ix, iy];
				}

				//Complex[] resFFT = FFT (tmpx);
				Complex[] resFFT = SwapElements (tmpx);
				FFTBody (resFFT, inverse);

				for (int ix = 0; ix < nx; ++ix)
				{
					res[ix, iy] = resFFT[ix];
				}
			}

			//    Y
			Complex[] tmpy = new Complex[ny];
			for (int ix = 0; ix < nx; ++ix)
			{
				for (int iy = 0; iy < ny; ++iy)
				{
					tmpy[iy] = res[ix, iy];
				}

				//Complex[] resFFT = FFT (tmpy);
				Complex[] resFFT = SwapElements (tmpy);
				FFTBody (resFFT, inverse);

				for (int iy = 0; iy < ny; ++iy)
				{
					res[ix, iy] = resFFT[iy];
				}
			}
			return res;
		}

		/// <summary>
		///    
		/// </summary>
		/// <param name="input"> </param>
		/// <returns> </returns>
		static public Complex[] FFT (Complex[] input)
		{
			Complex[] res = SwapElements(input);

			FFTBody(res, false);

			return res;
		}

		/// <summary>
		///    
		/// </summary>
		/// <param name="input"> </param>
		/// <returns> </returns>
		static public Complex[] FFT (Double[] input)
		{
			Complex[] res = SwapElements(input);

			FFTBody(res, false);

			return res;
		}

		/// <summary>
		///    
		/// </summary>
		/// <param name="input"> </param>
		/// <returns> </returns>
		static public Complex[] IFFT (Complex[] input)
		{
			Complex[] res = SwapElements(input);

			FFTBody(res, true);

			return res;
		}

		/// <summary>
		///    
		/// </summary>
		/// <param name="input"> </param>
		/// <returns> </returns>
		static public Complex[] IFFT (Double[] input)
		{
			Complex[] res = SwapElements(input);

			FFTBody(res, true);

			return res;
		}

		/// <summary>
		///      . 
		/// </summary>
		/// <param name="res">,      .
		///    </param>
		private static void FFTBody(Complex[] res, bool inverse)
		{
			Int32 Size = res.Length;

			//   
			Int32 lg;
			for (lg = 0; Size >> lg > 1; lg++);
	
			for (Int32 s = 1; s <= lg; ++s)
			{
				Int32 m = 1 << s;
				Complex Wm = inverse ? 
					new Complex(Math.Cos(2.0 * Math.PI / m), Math.Sin(2.0 * Math.PI / m)) : 
					new Complex(Math.Cos(2.0 * Math.PI / m), Math.Sin( -2.0 * Math.PI / m));

				Complex W = 1.0;

				Complex t = new Complex();
				//Complex u;

				for (Int32 j = 0; j < m / 2; ++j)
				{
					for (Int32 k = j; k < Size; k += m)
					{
						t = new Complex(W.Re * res[k + m / 2].Re - W.Im * res[k + m / 2].Im,
						                W.Re * res[k + m / 2].Im + W.Im * res[k + m / 2].Re);
						Complex u = new Complex(res[k].Re, res[k].Im);

						res[k].Re = u.Re + t.Re;
						res[k].Im = u.Im + t.Im;

						/*res[k + m / 2].Re = u.Re - t.Re;
						res[k + m / 2].Im = u.Im - t.Im;*/
                                                res[k + m / 2] = u - t;

					}

					t.Re = W.Re * Wm.Re - W.Im * Wm.Im;
					t.Im = W.Re * Wm.Im + W.Im * Wm.Re;

					W = t;
				}
			}

			if (inverse)
			{
				int count = res.Length;
				for (int i = 0; i < count; ++i)
				{
					res[i] /= count;
				}
			}
		}

		/// <summary>
		///    
		/// </summary>
		/// <param name="input"> </param>
		/// <returns> </returns>
		private static Complex[] SwapElements (Complex[] input)
		{
			Int32 Size = input.Length;

			//   
			Int32 Lg;
			for (Lg = 0; Size >> Lg > 1; Lg++);

			Complex[] res = new Complex[Size];

			for (Int32 i = 0; i < Size; ++i)
			{
				Int32 k = 0;
				for (Int32 j = 0; j < Lg; ++j)
				{
					if ((i & (1 << j)) == (1 << j))
					{
						k += 1 << (Lg - j - 1);
					}
				}

				/*res[i].Re = input[k].Re;
				res[i].Im = input[k].Im;*/

				res[i] = input[k];
			}

			return res;
		}

		/// <summary>
		///    
		/// </summary>
		/// <param name="input"> </param>
		/// <returns> </returns>
		private static Complex[] SwapElements (Double[] input)
		{
			Int32 Size = input.Length;

			//   
			Int32 Lg;
			for (Lg = 0; Size >> Lg > 1; Lg++);

			Complex[] res = new Complex[Size];

			for (Int32 i = 0; i < Size; ++i)
			{
				Int32 k = 0;
				for (Int32 j = 0; j < Lg; ++j)
				{
					if ((i & (1 << j)) == (1 << j))
					{
						k += 1 << (Lg - j - 1);
					}
				}

				/*res[i].Re = input[k].Re;
				res[i].Im = input[k].Im;*/

				res[i] = input[k];
			}

			return res;
		}

		/// <summary>
		///   .  
		/// </summary>
		/// <param name="input"> </param>
		/// <returns> </returns>
		static public Complex[] DFT (Complex[] input)
		{
			Complex[] y = DFTBody(input, false);
			
			return y;
		}

		static public Complex[] DFT (double[] input)
		{
			Complex[] y = DFTBody(input, false);

			return y;
		}

		static public Complex[] IDFT (double[] input)
		{
			Complex[] y = DFTBody(input, true);

			return y;
		}

		static public Complex[] IDFT (Complex[] input)
		{
			Complex[] y = DFTBody(input, true);

			return y;
		}

		private static Complex[] DFTBody (Complex[] input, bool inverse)
		{
			Int32 Count = input.Length;
			Complex[] y = new Complex[Count];

			int kn = inverse ? 1 : -1;

			for (Int32 k = 0; k < Count; ++k)
			{
				//y[k] = 0;
				Complex tmp = new Complex(0);

				for (Int32 j = 0; j < Count; ++j)
				{
					Complex n = ((kn * 2.0 * Math.PI * k * j) / Count) * Complex.J;
					tmp += input[j] * Complex.Exp(n);
				}

				if (inverse)
				{
					// 
					y[k] = tmp / Count;
				}
				else
				{
					// 
					y[k] = tmp ;
				}
			}
			return y;
		}

		private static Complex[] DFTBody (double[] input, bool inverse)
		{
			Int32 Count = input.Length;
			Complex[] y = new Complex[Count];

			int kn = inverse ? 1 : -1;

			for (Int32 k = 0; k < Count; ++k)
			{
				//y[k] = 0;
				Complex tmp = new Complex(0);

				for (Int32 j = 0; j < Count; ++j)
				{
					Complex n = ((kn * 2.0 * Math.PI * k * j) / Count) * Complex.J;
					tmp += input[j] * Complex.Exp(n);
				}

				if (inverse)
				{
					// 
					y[k] = tmp / Count;
				}
				else
				{
					// 
					y[k] = tmp;
				}
			}
			return y;
		}

		
		static public T[] fftShift<T>(T[] input)
		{
			int count = input.Length;
			T[] res = new T[count];
			int c2 = count / 2;

			for (int i = 0; i < c2; ++i)
			{
				res[i] = input[c2 + i];
				res[count - i - 1] = input[c2 - i - 1];
			}

			return res;
		}

		static public T[,] fftShift<T> (T[,] input, int width)
		{
			int height = input.Length / width;

			int w2 = width / 2;
			int h2 = height / 2;

			T[,] res = new T[width, height];

			for (int i = 0; i < w2; ++i)
			{
				for (int j = 0; j < h2; ++j)
				{
					//  1  4 
					res[i, j] = input[w2 + i, h2 + j];
					res[width - i - 1, height - j - 1] =
						input[w2 - i - 1, h2 - j - 1];

					//  2  3 
					res[i, height - j - 1] =
						input[w2 + i, h2 - j - 1];
					res[width - i - 1, j] =
						input[w2 - i - 1, h2 + j];
				}
			}

			return res;
		}
	}
}
