using System;
using System.Text;

using NUnit.Framework;

using Jenyay.Mathematics;

namespace MathematicsTest
{
	/// <summary>
	///   
	/// </summary>
	[TestFixture]
	public class FourierTest
	{
		public FourierTest()
		{
		}

		[Test]
		public void FFTTest1()
		{
			const Int32 Size1 = 256;
			Int32 i;

			Complex[] arr1 = new Complex[Size1];
			for (i = 0; i < Size1; ++i)
			{
				if ((i % 2) == 0)
				{
					arr1[i] = new Complex(1.0);
				}
				else
				{
					arr1[i] = new Complex(-1.0);
				}
			}

			Complex[] res = Fourier.FFT(arr1);
			Assert.IsTrue(res[63].Re - 1.0 < 0.0001 && res[63].Im == 0.0);
			for (i = 0; i < Size1 / 2 - 1; ++i)
			{
				String test = String.Format("i = {0}    c = {1}", i, res[i]).ToString();
				Assert.IsTrue (res[i].Re == 0.0 && res[1].Im == 0.0, test);
			}
		}

		[Test]
		public void FFTTest2()
		{
			const Int32 Size1 = 256;
			Int32 i;

			Double[] arr1 = new Double[Size1];
			for (i = 0; i < Size1; ++i)
			{
				if ((i % 2) == 0)
				{
					arr1[i] = 1.0;
				}
				else
				{
					arr1[i] = -1.0;
				}
			}

			Complex[] res = Fourier.FFT(arr1);
			Assert.IsTrue(res[63].Re - 1.0 < 0.0001 && res[63].Im == 0.0);
			for (i = 0; i < Size1 / 2 - 1; ++i)
			{
				String test = String.Format("i = {0}    c = {1}", i, res[i]).ToString();
				Assert.IsTrue (res[i].Re == 0.0 && res[1].Im == 0.0, test);
			}
		}

		/*[Test]
		public void DFTTest0 ()
		{
			Int32 Size1 = 258;

			double[] arr1 = new double[Size1];
			for (int i = 0; i < Size1; ++i)
			{
				if ((i % 2) == 0)
				{
					arr1[i] = 1.0;
				}
				else
				{
					arr1[i] = -1.0;
				}
			}

			Complex[] res = Fourier.DFT(arr1);
			Assert.IsTrue(Math.Abs(res[Size1 / 2].Re - 1.0) < 0.0001 && 
				Math.Abs(res[Size1 / 2].Im) < 0.0001);
			for (int i = 0; i < Size1; ++i)
			{
				String test = String.Format("i = {0}    c = {1}", i, res[i]).ToString();

				if (i != Size1 / 2)
				{
					Assert.IsTrue(test, Math.Abs(res[i].Re) < 0.0001 && Math.Abs(res[i].Im) < 0.0001);
				}
			}
		}*/

		/*[Test]
		public void DFTTest1()
		{
			Int32 Size1 = 258;

			Complex[] arr1 = new Complex[Size1];
			for (int i = 0; i < Size1; ++i)
			{
				if ((i % 2) == 0)
				{
					arr1[i] = 1.0;
				}
				else
				{
					arr1[i] = -1.0;
				}
			}

			Complex[] res = Fourier.DFT(arr1);
			Assert.IsTrue (Math.Abs (res[Size1 / 2].Re - 1.0) < 0.0001 && 
				Math.Abs(res[Size1 / 2].Im) < 0.0001);
			for (int i = 0; i < Size1; ++i)
			{
				String test = String.Format("i = {0}    c = {1}", i, res[i]).ToString();

				if (i != Size1 / 2)
				{
					Assert.IsTrue(test, Math.Abs(res[i].Re) < 0.0001 && Math.Abs(res[i].Im) < 0.0001);
				}
			}
		}
		 */

		/// <summary>
		///  DTF  FFT (Complex VS Complex)
		/// </summary>
		[Test]
		public void DFTTest2 ()
		{
			Int32 count = 256;

			Complex[] arr = new Complex[count];
			for (int i = 0; i < count; ++i)
			{
				arr[i] = i * 0.1;
			}

			Complex[] sp_fft = Fourier.FFT(arr);
			Complex[] sp_dft = Fourier.FFT(arr);

			for (int i = 0; i < count; ++i)
			{
				String text = String.Format("{0}) fft = {1}   dft={2}",
					i, sp_fft[i], sp_dft[i]);

				Assert.IsTrue (Math.Abs (sp_fft[i].Re - sp_dft[i].Re) < 0.0001, text);
				Assert.IsTrue (Math.Abs (sp_fft[i].Im - sp_dft[i].Im) < 0.0001, text);
			}
		}

		/// <summary>
		///  DTF  FFT (double VS double)
		/// </summary>
		[Test]
		public void DFTTest3 ()
		{
			Int32 count = 256;

			double[] arr = new double[count];
			for (int i = 0; i < count; ++i)
			{
				arr[i] = i * 0.1;
			}

			Complex[] sp_fft = Fourier.FFT(arr);
			Complex[] sp_dft = Fourier.DFT(arr);

			for (int i = 0; i < count; ++i)
			{
				String text = String.Format("{0}) fft = {1}   dft={2}",
					i, sp_fft[i], sp_dft[i]);

				Assert.IsTrue (Math.Abs (sp_fft[i].Re - sp_dft[i].Re) < 0.0001, text);
				Assert.IsTrue (Math.Abs (sp_fft[i].Im - sp_dft[i].Im) < 0.0001, text);
			}
		}

		/// <summary>
		///  DTF  IDFT (Complex)
		/// </summary>
		[Test]
		public void DFTTest4 ()
		{
			Int32 count = 256;

			Complex[] arr = new Complex[count];
			for (int i = 0; i < count; ++i)
			{
				arr[i] = i * 0.1;
			}

			Complex[] sp_dft = Fourier.DFT(arr);
			Complex[] sign_idft = Fourier.IDFT(sp_dft);

			for (int i = 0; i < count; ++i)
			{
				String text = String.Format("{0}) arr = {1}   idft={2}",
					i, arr[i], sign_idft[i].Abs);

				Assert.IsTrue (Math.Abs (sign_idft[i].Abs - arr[i].Abs) < 0.0001, text);
			}
		}

		[Test]
		public void FFT2Test1 ()
		{
			int count = 4;
			double error = 0.0001;
			double[,] input = new double[count, count];

			for (int i = 0; i < count; ++i)
			{
				for (int j = 0; j < count; ++j)
				{
					input[i, j] = 0;
				}
			}

			input[count / 2, count / 2] = 1;


			//////////////////////////////////////////////////////////////////////////
			//    

			Complex[,] res = Fourier.FFT2 (input, count);

			for (int i = 0; i < count; ++i)
			{
				for (int j = 0; j < count; ++j)
				{
					StringBuilder sb = new StringBuilder();
					string text = sb.AppendFormat ("res[{0}, {1}] = {2}", i, j, res[i, j]).ToString();

					Assert.IsTrue (Math.Abs (res[i, j].Im) < error, text);

					if ( (i + j) % 2 == 0)
					{
						Assert.IsTrue (Math.Abs (res[i, j].Re - 1) < error, text);
					}
					else
					{
						Assert.IsTrue (Math.Abs (res[i, j].Re + 1) < error, text);
					}
				}
			}

			//////////////////////////////////////////////////////////////////////////
			//    

			Complex[,] ires = Fourier.IFFT2 (res, count);
			for (int i = 0; i < count; ++i)
			{
				for (int j = 0; j < count; ++j)
				{
					StringBuilder sb = new StringBuilder ();
					string text = sb.AppendFormat ("ires[{0}, {1}] = {2}", i, j, ires[i, j]).ToString ();

					Assert.IsTrue (Math.Abs (ires[i, j].Im) < error, text);

					if (i == count / 2 && j == count / 2)
					{
						Assert.IsTrue (Math.Abs (ires[i, j].Re - 1) < error, text);
					}
					else
					{
						Assert.IsTrue (Math.Abs (ires[i, j].Re) < error, text);
					}
				}
			}
		}

		/// <summary>
		///   
		/// </summary>
		[Test]
		public void ParsevalTest ()
		{
			Int32 count = 256;

			Complex[] arr = new Complex[count];
			for (int i = 0; i < count; ++i)
			{
				arr[i] = i * 0.1;
			}

			Complex[] spectrum_fft = Fourier.FFT(arr);
			Complex[] spectrum_dft = Fourier.DFT(arr);

			double sum1 = 0;
			double sum2 = 0;
			double sum3 = 0;

			for (int i = 0; i < count; ++i)
			{
				sum1 += arr[i].Abs * arr[i].Abs;
				sum2 += spectrum_fft[i].Abs * spectrum_fft[i].Abs;
				sum3 += spectrum_dft[i].Abs * spectrum_dft[i].Abs;
			}

			sum2 /= count;
			sum3 /= count;

			string text1 = String.Format ("sum1 = {0}   sum2 = {1}",
				sum1, sum2);

			string text2 = String.Format ("sum1 = {0}   sum3 = {1}",
				sum1, sum3);

			Assert.IsTrue (Math.Abs (sum1 - sum2) < 0.0001, text1);
			Assert.IsTrue (Math.Abs (sum1 - sum3) < 0.0001, text2);
		}

		[Test]
		public void fftShiftTest1 ()
		{
			int count = 8;

			double[] src = new double[count];

			for (int i = 0; i < count; ++i)
			{
				src[i] = i * 0.1;
			}

			double[] shift = Fourier.fftShift<double>(src);

			Assert.IsTrue(Math.Abs (shift[0]- 0.4) < 0.0001);
			Assert.IsTrue(Math.Abs(shift[1] - 0.5) < 0.0001);
			Assert.IsTrue(Math.Abs(shift[2] - 0.6) < 0.0001);
			Assert.IsTrue(Math.Abs(shift[3] - 0.7) < 0.0001);
			Assert.IsTrue(Math.Abs(shift[4] - 0.0) < 0.0001);
			Assert.IsTrue(Math.Abs(shift[5] - 0.1) < 0.0001);
			Assert.IsTrue(Math.Abs(shift[6] - 0.2) < 0.0001);
			Assert.IsTrue(Math.Abs(shift[7] - 0.3) < 0.0001);
		}

		[Test]
		public void fftShiftTest2 ()
		{
			int count = 8;

			double[] src = new double[count];

			for (int i = 0; i < count; ++i)
			{
				src[i] = i * 0.1;
			}

			double[] shift = Fourier.fftShift<double>(src);
			double[] ishift = Fourier.fftShift<double>(shift);


			for (int i = 0; i < count; ++i)
			{
				Assert.IsTrue (Math.Abs (ishift[i] - src[i]) < 0.0001, i.ToString ());
			}
		}

		[Test]
		public void DFT2Test1 ()
		{
			int nx = 4;
			int ny = 4;

			Complex[,] inp = new Complex[nx, ny];

			for (int i = 0; i < nx; ++i)
			{
				for (int j = 0; j < ny; ++j)
				{
					inp[i, j] = (double)i * j - j;
				}
			}

			Complex[,] resFFT = Fourier.FFT2(inp, nx);
			Complex[,] resDFT = Fourier.DFT2(inp, nx);

			for (int i = 0; i < nx; ++i)
			{
				for (int j = 0; j < ny; ++j)
				{
					string text = String.Format ("[{0}, {1}] {2} VS {3}",
						i, j, resFFT[i, j], resDFT[i, j]);

					Assert.IsTrue (Math.Abs (resFFT[i, j].Re - resDFT[i, j].Re) < 0.0001, text);
					Assert.IsTrue (Math.Abs (resFFT[i, j].Im - resDFT[i, j].Im) < 0.0001, text);
				}
			}
		}

		[Test]
		public void DFT2Test2 ()
		{
			int nx = 4;
			int ny = 4;

			Complex[,] inp = new Complex[nx, ny];

			for (int i = 0; i < nx; ++i)
			{
				for (int j = 0; j < ny; ++j)
				{
					inp[i, j] = (double)i * j - j;
				}
			}

			Complex[,] resDFT = Fourier.DFT2(inp, nx);
			Complex[,] resIDFT = Fourier.IDFT2(resDFT, nx);

			for (int i = 0; i < nx; ++i)
			{
				for (int j = 0; j < ny; ++j)
				{
					string text = String.Format("[{0}, {1}] {2} VS {3}",
						i, j, inp[i, j], resIDFT[i, j]);

					Assert.IsTrue (Math.Abs (inp[i, j].Re - resIDFT[i, j].Re) < 0.0001, text);
				}
			}
		}
	}
}
