using System;
using System.Text;

using NUnit.Framework;

using Jenyay.Mathematics;

namespace MathematicsTest
{
	[TestFixture]
	public class SplineTest
	{
		Spline _spline;

		[SetUp]
		public void Init ()
		{
			double[] x = new double[7];
			double[] y = new double[7];

			x[0] = 0;
			y[0] = 0;

			x[1] = 0.3;
			y[1] = 0.299401;

			x[2] = 0.5;
			y[2] = 0.4923442;

			x[3] = 0.8;
			y[3] = 0.7228442;

			x[4] = 1;
			y[4] = 0.7798934;

			x[5] = 1.3;
			y[5] = 0.6385505;

			x[6] = 1.5;
			y[6] = 0.4452612;

			_spline = new Spline (x, y);
		}

		/// <summary>
		///   
		/// </summary>
		[Test]
		public void TestDerivative ()
		{
			double[] m = new double[7];
			m[0] = 0;
			m[1] = -0.08877862525;
			m[2] = -0.5547268737;
			m[3] = -2.019378004;
			m[4] = -3.563639669;
			m[5] = -1.902729099;
			m[6] = 0;

			double[] derivative = _spline.Derivative;
			for (int i = 0; i < 7; i++)
			{
				Assert.Less (Math.Abs (derivative[i] - m[i]), 1e-9);
			}
		}

		[Test]
		public void TestApproximate ()
		{
			Assert.Less (Math.Abs (_spline.GetValue (0.2) - 0.2000938813), 1e-10, 
				_spline.GetValue (0.2).ToString());

			Assert.Less (Math.Abs (_spline.GetValue (0.48) - 0.4737408625), 1e-10,
				_spline.GetValue (0.48).ToString ());

			Assert.Less (Math.Abs (_spline.GetValue (1.6) - 0.3422741197), 1e-10,
				_spline.GetValue (1.6).ToString ());

			Assert.Less (Math.Abs (_spline.GetValue (0) - 0), 1e-10,
				_spline.GetValue (0).ToString ());

			Assert.Less (Math.Abs (_spline.GetValue (0.5) - 0.4923442), 1e-10,
				_spline.GetValue (0.5).ToString ());
		}
	}
}
